/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: login-dual.c,v 1.1 2004/01/08 13:28:28 who_ami Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 *
 * An example of use dual connections to Tlen server.
 *
 *
 *
 */

#include <stdio.h>

#include "libtlen2.h"


GMainLoop *mainloop;

void event_cb (TlenConnection *tc);

int main (int argc, char **argv)
{
  TlenConnection *tc1, *tc2;

  if (argc != 5)
  {
    printf("usage: %s <login1> <password1> <login2> <password2>\n", argv[0]);
    return 1;
  }
  tlen_init ();
  tlen_debug_set (TLEN_DEBUG_HIGH);
  tc1 = tlen_connection_new (argv[1], argv[2]);
  tlen_connection_open (tc1, event_cb);
  tc2 = tlen_connection_new (argv[3], argv[4]);
  tlen_connection_open (tc2, event_cb);
  mainloop = g_main_loop_new (NULL, FALSE);
  g_main_loop_run (mainloop);
  tlen_connection_delete (tc1);
  tlen_connection_delete (tc2);
  return 0;
}

void event_cb (TlenConnection *tc)
{
  TlenEvent *ev;
  TlenRosterItem *item;
  GSList *l;

  if (tc)
  {
    printf ("Event for user %s\n", tc->username);
    while ((ev = tlen_event_get (tc)))
    {
      switch (ev->type)
      {
        case TLEN_EVENT_MESSAGE:
          printf ("Event message from: '%s', type: %d body: '%s', stamp: '%s'\n",
                  TLEN_EVENT_MESSAGE (ev)->from,
                  TLEN_EVENT_MESSAGE (ev)->type,
                  TLEN_EVENT_MESSAGE (ev)->body,
                  TLEN_EVENT_MESSAGE (ev)->stamp);
          break;
        case TLEN_EVENT_PRESENCE:
          printf ("Event presence from: '%s', status: %d description: '%s'\n",
                  TLEN_EVENT_PRESENCE (ev)->from,
                  TLEN_EVENT_PRESENCE (ev)->status,
                  TLEN_EVENT_PRESENCE (ev)->description);
          break;
        case TLEN_EVENT_CONNECTED:
          printf ("Event connected\n");
//          g_main_loop_quit (mainloop);
          break;
        case TLEN_EVENT_CONNECTION_FAILED:
          printf ("Event connection failed\n");
          g_main_loop_quit (mainloop);
          break;
        case TLEN_EVENT_DISCONNECTED:
          printf ("Event disconnected\n");
          g_main_loop_quit (mainloop);
          break;
        case TLEN_EVENT_UNAUTHORIZED:
          printf ("Unauthorized event.\n");
          g_main_loop_quit(mainloop);
          break;
        case TLEN_EVENT_LOGGED:
          printf ("Logged in!\n");
          tlen_roster_get (tc);
//          g_main_loop_quit(mainloop);
          break;
        case TLEN_EVENT_ROSTER:
          printf ("Roster event.\n");
          l = TLEN_EVENT_ROSTER (ev)->list;
          while (l)
          {
            item = (TlenRosterItem*) l->data;
            if (item)
            {
              printf ("%s : %s : %s : %s\n", item->jid, item->name,
                      item->group, item->subscription);
            }
            l = l->next;
          }
          tlen_presence (tc, TLEN_PRESENCE_AVAILABLE, "Libtlen2 test");
          break;
        default:
          printf ("event not recognized!\n");
          g_main_loop_quit (mainloop);
          break;
      }
      tlen_event_free (ev);
    }
  }
}

