/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: libtlen2-events.c,v 1.18 2004/05/29 17:25:36 mati Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#include "libtlen2-events.h"
#include "libtlen2-p2p.h"

/**
  * This is to be used if there is no memory
  * or the tlen protocol has gone mad.
  **/
TlenEvent _tlen_event_critical_failure = {NULL, TLEN_EVENT_CRITICAL_FAILURE};


/**
  * tlen_event_new:
  * @type: event type to create.
  *
  * Create new event.
  *
  * returns: new event of type @type or NULL if error occured.
  **/
TlenEvent *tlen_event_new (TlenEventType type)
{
  TlenEvent *ev = NULL;

  if (type == TLEN_EVENT_CRITICAL_FAILURE)
  {
    ev = &_tlen_event_critical_failure;
  }
  else
  {
    ev = g_new0 (TlenEvent, 1);
    /* if you allocate more memory remember to free it in tlen_event_free */
    if (ev)
    {
      ev->type = type;
      switch(type)
      {
        case TLEN_EVENT_P2P:
          ev->event = g_new0 (TlenEventP2p, 1);
          break;
        case TLEN_EVENT_FILE:
          ev->event = g_new0 (TlenEventFile, 1);
          break;
        case TLEN_EVENT_PUBDIR:
          ev->event = g_new0 (TlenEventPubdir, 1);
          if (ev->event)
            TLEN_EVENT_PUBDIR(ev)->item = g_new0 (TlenPubdirItem, 1);
          break;
        case TLEN_EVENT_NEWMAIL:
        case TLEN_EVENT_WEBMSG:
        case TLEN_EVENT_MESSAGE:
          ev->event = g_new0 (TlenEventMessage, 1);
          break;
        case TLEN_EVENT_ROSTER:
          ev->event = g_new0 (TlenEventRoster, 1);
          break;
        case TLEN_EVENT_SEARCH:
          ev->event = g_new0 (TlenEventSearch, 1);
          break;
        case TLEN_EVENT_PRESENCE:
          ev->event = g_new0 (TlenEventPresence, 1);
          break;
        case TLEN_EVENT_ADVERT:
          ev->event = g_new0 (TlenEventAdvert, 1);
          break;
        case TLEN_EVENT_NOTIFY:
          ev->event = g_new0 (TlenEventNotify, 1);
          break;
        default:
          break;
      }
    }
  }
  return ev;
}

/**
  * tlen_event_free:
  * @ev: event to free.
  *
  * Free event memory.
  **/
void tlen_event_free (TlenEvent *ev)
{
  if (ev && (ev->type != TLEN_EVENT_CRITICAL_FAILURE))
  {
    switch (ev->type)
    {
      case TLEN_EVENT_P2P:
        g_free (TLEN_EVENT_P2P (ev)->i);
        g_free (TLEN_EVENT_P2P (ev));
        break;
      case TLEN_EVENT_FILE:
        _tlen_p2p_free (TLEN_EVENT_FILE (ev)->p);
        g_free (TLEN_EVENT_FILE (ev));
        break;
      case TLEN_EVENT_PUBDIR:
        tlen_pubdir_item_free (TLEN_EVENT_PUBDIR (ev)->item);
        g_free (TLEN_EVENT_PUBDIR (ev));
        break;
      case TLEN_EVENT_SEARCH:
        {
          GSList *l;
          TlenPubdirItem *r;
          l = TLEN_EVENT_SEARCH (ev)->list;
          while (l)
          {
            r = (TlenPubdirItem*) (l->data);
            if (r)
              tlen_pubdir_item_free (r);
            l = l->next;
          }
          g_slist_free (TLEN_EVENT_SEARCH (ev)->list);
          g_free (TLEN_EVENT_SEARCH (ev));
        }
        break;
      case TLEN_EVENT_ROSTER:
        {
          GSList *l;
          TlenRosterItem *r;
          l = TLEN_EVENT_ROSTER (ev)->list;
          while (l)
          {
            r = (TlenRosterItem*) (l->data);
            if (r)
            {
              g_free (r->jid);
              g_free (r->subscription);
              g_free (r->name);
              g_free (r->group);
            }
            g_free (r);
            l = l->next;
          }
          g_slist_free (TLEN_EVENT_ROSTER (ev)->list);
          g_free (TLEN_EVENT_ROSTER (ev));
        }
        break;
      case TLEN_EVENT_NEWMAIL:
      case TLEN_EVENT_WEBMSG:
      case TLEN_EVENT_MESSAGE:
        if (ev->event)
        {
          g_free (TLEN_EVENT_MESSAGE (ev)->from);
          g_free (TLEN_EVENT_MESSAGE (ev)->body);
          g_free (TLEN_EVENT_MESSAGE (ev)->e);
          g_free (TLEN_EVENT_MESSAGE (ev)->stamp);
          g_free (TLEN_EVENT_MESSAGE (ev));
        }
        break;
      case TLEN_EVENT_PRESENCE:
        if (ev->event)
        {
          g_free (TLEN_EVENT_PRESENCE (ev)->from);
          g_free (TLEN_EVENT_PRESENCE (ev)->description);
          g_free (TLEN_EVENT_PRESENCE (ev));
        }
        break;
      case TLEN_EVENT_ADVERT:
        if (ev->event)
        {
          g_free (TLEN_EVENT_ADVERT (ev)->url);
          g_free (TLEN_EVENT_ADVERT (ev)->name);
          g_free (TLEN_EVENT_ADVERT (ev));
        }
        break;
      case TLEN_EVENT_NOTIFY:
        if (ev->event)
        {
          g_free (TLEN_EVENT_NOTIFY (ev)->from);
          g_free (TLEN_EVENT_NOTIFY (ev));
        }
        break;
      case TLEN_EVENT_CONNECTED:
      case TLEN_EVENT_PUBDIR_CHANGED:
      case TLEN_EVENT_UNKNOWN:
      case TLEN_EVENT_DISCONNECTED:
      case TLEN_EVENT_CONNECTION_FAILED:
      case TLEN_EVENT_UNAUTHORIZED:
      default:
        break;
    }
    g_free (ev);
  }
}

/**
  * tlen_event_critical_failure:
  * @tc: #TlenConnection to use.
  *
  * Sends TLEN_EVENT_CRITICAL_FAILURE to application.
  *
  **/
void tlen_event_critical_failure(TlenConnection *tc)
{
  if (!tc)
  {
    tlen_debug("Error in error handling is too much. Exiting.\n");
    exit(20);
  }
  tc->critical_failure = TRUE;
  tc->event_cb(tc);
}


/**
  * tlen_event_add:
  * @tc: #TlenConnection to use.
  * @event: event to add.
  *
  * Add new event to the end of events lists associated with @tc.
  *
  * returns: TRUE if event was added succesfully, FALSE otherwise.
  **/
gboolean tlen_event_add (TlenConnection *tc, TlenEvent *event)
{
  if (!tc)
  {
    tlen_debug ("tc == NULL!\n");
    return FALSE;
  }
  if (!event)
  {
    tlen_debug ("event == NULL!\n");
    return FALSE;
  }
  tc->events_list = g_slist_append (tc->events_list, event);
  if (tc->event_cb)
    tc->event_cb (tc);
  return TRUE;
}

/**
  * tlen_event_clear:
  * @tc: #TlenConnection to use.
  *
  * Remove all events from the events list associated with @tc.
  *
  * returns: TRUE if list was cleared, FALSE otherwise.
  **/
gboolean tlen_event_clear (TlenConnection *tc)
{
  GSList *l;
  TlenEvent *ev;

  if (!tc)
  {
    tlen_debug ("tc == NULL!\n");
    return FALSE;
  }
  l = tc->events_list;
  while (l)
  {
    ev = (TlenEvent*) l->data;
    tlen_event_free (ev);
    l = l->next;
  }
  if (tc->events_list)
  {
    g_slist_free (tc->events_list);
    tc->events_list = NULL;
  }
  return TRUE;
}

/**
  * tlen_event_get:
  * @tc: #TlenConnection to use.
  *
  * Get an event from the events list associated with @tc.
  *
  * returns: new event or NULL if there is no events.
  **/
TlenEvent *tlen_event_get (TlenConnection *tc)
{
  TlenEvent *ev = NULL;

  if (!tc)
  {
    tlen_debug ("tc == NULL!\n");
  }
  else
  {
    if (tc->critical_failure)
    {
      /* This kind of event requires special handling. */
      ev = &_tlen_event_critical_failure;
    }
    else
    {
      if (tc->events_list)
      {
        ev = (TlenEvent*) tc->events_list->data;
        if (ev)
        {
          tc->events_list = g_slist_remove (tc->events_list, (gconstpointer) ev);
        }
      }
    }
  }
  return ev;
}

