/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: libtlen2-login.c,v 1.4 2003/12/29 12:46:07 who_ami Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#include "libtlen2-login.h"
#include "libtlen2-utils.h"
#include <string.h>


static
gchar *_calc_passcode (const gchar *pass);

gboolean _tlen_auth_send (TlenConnection *tc)
{
  gchar *s;
  gsize bytes;
  GSHA *sha;
  gchar *digest;
  gchar *passcode;
  gboolean ret;

  if (!tc)
  {
    tlen_debug ("tc == NULL!\n");
    return FALSE;
  }
  passcode = _calc_passcode (tc->password);
  digest = g_strdup_printf ("%s%s", tc->sid, passcode);
  g_free (passcode);
  sha = gnet_sha_new (digest, tlen_strlen (digest));
  g_free (digest);
  digest = gnet_sha_get_string (sha);
  gnet_sha_delete (sha);

  s = g_strdup_printf ("<iq type='set' id='%s'><query xmlns='jabber:iq:auth'><username>%s</username><digest>%s</digest><resource>t</resource></query></iq>",
                tc->sid, tc->username, digest);
  ret = tlen_connection_socket_writen (tc, s, tlen_strlen (s), &bytes);
  g_free (s);
  g_free (digest);
  return ret;
}

static
gchar *_calc_passcode (const gchar *pass)
{
  gint magic1 = 0x50305735, magic2 = 0x12345671, sum = 7;
  gchar z;

  if (!pass)
    return NULL;
  while ((z = *pass++) != 0)
  {
    if (z == ' ')
      continue;
    if (z == '\t')
      continue;
    magic1 ^= (((magic1 & 0x3f) + sum) * z) + (magic1 << 8);
    magic2 += (magic2 << 8) ^ magic1;
    sum += z;
  }
  magic1 &= 0x7fffffff;
  magic2 &= 0x7fffffff;
  return g_strdup_printf ("%08x%08x", magic1, magic2);
}

gboolean
_tlen_login (TlenConnection *tc)
{
  gsize bytes_n;
  if (!tc)
  {
    tlen_debug ("tc == NULL!\n");
    return FALSE;
  }
  if (tc->state != TLEN_STATE_CONNECTED)
  {
    tlen_debug ("Not connected with the server.\n");
    return FALSE;
  }
  tc->state = TLEN_STATE_LOGGING;
  if (tlen_connection_socket_writen (tc, "<s v='2'>", strlen ("<s v='2'>"),
                                     &bytes_n))
  {
    return FALSE;
  }
  return TRUE;
}




