/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: libtlen2-roster.c,v 1.5 2004/02/23 14:51:32 who_ami Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#include "libtlen2-roster.h"
#include <string.h>

/**
  * tlen_unsubscribe_accept:
  * @tc: connection to use.
  * @id: id in form login or login(at)tlen.pl
  *
  * Accept unsubscription.
  *
  * returns: TRUE if request was sent, FALSE  otherwise
  **/
gboolean tlen_unsubscribe_accept (TlenConnection *tc, const gchar *id)
{
  gboolean ret;
  gchar *q, *login, *p;
  gsize b;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  p = strchr (id, '@');
  if (!p)
    login = g_strdup_printf ("%s@tlen.pl", id);
  else
    login = g_strdup (id);
  q = g_strdup_printf ("<presence to='%s' type='unsubscribed'/>", login);
  g_free (login);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &b);
  g_free (q);
  return ret;
}

/**
  * tlen_subscribe_accept:
  * @tc: connection to use.
  * @id: id in form login or login(at)tlen.pl
  *
  * Accept subscription.
  *
  * returns: TRUE if request was sent, FALSE  otherwise
  **/
gboolean tlen_subscribe_accept (TlenConnection *tc, const gchar *id)
{
  gboolean ret;
  gchar *q, *login, *p;
  gsize b;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  p = strchr (id, '@');
  if (!p)
    login = g_strdup_printf ("%s@tlen.pl", id);
  else
    login = g_strdup (id);
  q = g_strdup_printf ("<presence to='%s' type='subscribed'/>", login);
  g_free (login);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &b);
  g_free (q);
  return ret;
}

/**
  * tlen_subscribe_request:
  * @tc: connection to use.
  * @id: id in form login or login(at)tlen.pl
  *
  * Send a request for subscription.
  *
  * returns: TRUE if request was sent, FALSE  otherwise
  **/
gboolean tlen_subscribe_request (TlenConnection *tc, const gchar *id)
{
  gboolean ret;
  gchar *q, *login, *p;
  gsize b;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  p = strchr (id, '@');
  if (!p)
    login = g_strdup_printf ("%s@tlen.pl", id);
  else
    login = g_strdup (id);
  q = g_strdup_printf ("<presence to='%s' type='subscribe'/>", login);
  g_free (login);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &b);
  g_free (q);
  return ret;
}


/**
  * tlen_unsubscribe_request:
  * @tc: connection to use.
  * @id: id in form login or login(at)tlen.pl
  *
  * Send a request for unsubscription from user roster.
  *
  * returns: TRUE if request was sent, FALSE  otherwise
  **/
gboolean tlen_unsubscribe_request (TlenConnection *tc, const gchar *id)
{
  gboolean ret;
  gchar *q, *login, *p;
  gsize b;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  p = strchr (id, '@');
  if (!p)
    login = g_strdup_printf ("%s@tlen.pl", id);
  else
    login = g_strdup (id);
  q = g_strdup_printf ("<presence to='%s' type='unsubscribe'/>", login);
  g_free (login);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &b);
  g_free (q);
  return ret;
}

/**
  * tlen_contact_remove:
  * @tc: connection to use.
  * @id: id in form login or login(at)tlen.pl
  *
  * Remove user from your roster.
  *
  * returns: TRUE if request was sent, FALSE  otherwise
  **/
gboolean tlen_contact_remove (TlenConnection *tc, const gchar *id)
{
  gboolean ret;
  gchar *q, *login, *p;
  gsize b;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  p = strchr (id, '@');
  if (!p)
    login = g_strdup_printf ("%s@tlen.pl", id);
  else
    login = g_strdup (id);
  q = g_strdup_printf ("<iq type='set'><query xmlns='jabber:iq:roster'><item jid='%s' subscription='remove'></item></query></iq>",
                       login);
  g_free (login);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &b);
  g_free (q);
  return ret;
}

/**
  * tlen_contact_add:
  * @tc: connection to use.
  * @id: id in form login or login(at)tlen.pl
  *
  * Add user to your roster.
  *
  * returns: TRUE if request was sent, FALSE  otherwise
  **/
gboolean tlen_contact_add (TlenConnection *tc, const gchar *id,
                           const gchar *_name, const gchar *_group)
{
  gboolean ret;
  gchar *q, *login, *p;
  gsize b;
  gchar *name, *group;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!id)
  {
    tlen_debug ("id == NULL\n");
    return FALSE;
  }
  p = strchr (id, '@');
  if (!p)
    login = g_strdup_printf ("%s@tlen.pl", id);
  else
    login = g_strdup (id);
  if (_name)
    name = tlen_encode (_name);
  else
    name = NULL;
  if (_group)
    group = tlen_encode (_group);
  else
    group = NULL;
  if (name)
  q = g_strdup_printf ("<iq type='set'><query xmlns='jabber:iq:roster'><item jid='%s' name='%s'><group>%s</group></item></query></iq>",
                       login, name, group ? group : "");
  else
  q = g_strdup_printf ("<iq type='set'><query xmlns='jabber:iq:roster'><item jid='%s' ><group>%s</group></item></query></iq>",
                       login, group ? group : "");
  g_free (login);
  g_free (name);
  g_free (group);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &b);
  g_free (q);
  return ret;
}
