/* Libtlen2 - multiplatform Tlen.pl library
 * Copyright (C) 2003  Libtlen Team
 *
 * $Id: libtlen2-tuba.c,v 1.3 2004/02/27 14:12:15 who_ami Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#include "libtlen2-tuba.h"


/**
  * tlen_pubdir_search:
  * @tc: connection to use.
  * @item: pubdir item to search.
  *
  * Search public directory for people matching @item.
  *
  * returns: TRUE on success, FALSE otherwise.
  **/
gboolean tlen_pubdir_search (TlenConnection *tc, TlenPubdirItem *item)
{
  gchar *q, *s;
  gboolean ret;
  gsize bytes;
  GString *buf;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  if (!item)
  {
    tlen_debug ("item == NULL\n");
    return FALSE;
  }
  q = "<iq type='get' id='src' to='tuba'><query xmlns='jabber:iq:search'>";
  buf = g_string_new (q);
  s = tlen_encode (item->firstname);
  if (s)
  {
  g_string_append_printf (buf, "<first>%s</first>", s?s:"");
  g_free (s);
  }
  s = tlen_encode (item->lastname);
  if (s)
  {
  g_string_append_printf (buf, "<last>%s</last>", s?s:"");
  g_free (s);
  }
  s = tlen_encode (item->nick);
  if (s)
  {
  g_string_append_printf (buf, "<nick>%s</nick>", s?s:"");
  g_free (s);
  }
  s = tlen_encode (item->email);
  if (s)
  {
  g_string_append_printf (buf, "<email>%s</email>", s?s:"");
  g_free (s);
  }
  s = tlen_encode (item->id);
  if (s)
  {
  g_string_append_printf (buf, "<i>%s</i>", s?s:"");
  g_free (s);
  }
  s = tlen_encode (item->city);
  if (s)
  {
  g_string_append_printf (buf, "<c>%s</c>", s?s:"");
  g_free (s);
  }
  s = tlen_encode (item->school);
  if (s)
  {
  g_string_append_printf (buf, "<e>%s</e>", s?s:"");
  g_free (s);
  }
  if (item->gender)
  g_string_append_printf (buf, "<s>%d</s>", item->gender);
  if (item->status)
  g_string_append_printf (buf, "<m>%d</m>", item->status);
  if (item->age_min)
  g_string_append_printf (buf, "<d>%d</d>", item->age_min);
  if (item->age_max)
  g_string_append_printf (buf, "<u>%d</u>", item->age_max);
  if (item->job)
  g_string_append_printf (buf, "<j>%d</j>", item->job);
  if (item->look_for)
  g_string_append_printf (buf, "<r>%d</r>", item->look_for);
  if (item->voice)
  g_string_append_printf (buf, "<g>%d</g>", item->voice);
  if (item->plans)
  g_string_append_printf (buf, "<p>%d</p>", item->plans);
  buf = g_string_append (buf, "</query></iq>");
  q = g_string_free (buf, FALSE);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &bytes);
  g_free (q);
  return ret;
}

/**
  * tlen_pubdir_set:
  * @tc: connection to use
  *
  * Set public information about yourself.
  *
  * returns: TRUE on success, FALSE otherwise.
  */
gboolean tlen_pubdir_set (TlenConnection *tc, TlenPubdirItem *item)
{
  gchar *q, *s;
  gboolean ret;
  gsize bytes;
  GString *buf;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  q = "<iq type='set' id='tw' to='tuba'><query xmlns='jabber:iq:register'>";
  buf = g_string_new (q);
  s = tlen_encode (item->firstname);
  g_string_append_printf (buf, "<first>%s</first>", s?s:"");
  g_free (s);
  s = tlen_encode (item->lastname);
  g_string_append_printf (buf, "<last>%s</last>", s?s:"");
  g_free (s);
  s = tlen_encode (item->nick);
  g_string_append_printf (buf, "<nick>%s</nick>", s?s:"");
  g_free (s);
  s = tlen_encode (item->email);
  g_string_append_printf (buf, "<email>%s</email>", s?s:"");
  g_free (s);
  s = tlen_encode (item->city);
  g_string_append_printf (buf, "<c>%s</c>", s?s:"");
  g_free (s);
  s = tlen_encode (item->school);
  g_string_append_printf (buf, "<e>%s</e>", s?s:"");
  g_free (s);
  g_string_append_printf (buf, "<s>%d</s>", item->gender);
  g_string_append_printf (buf, "<v>%d</v>", item->visible);
  g_string_append_printf (buf, "<b>%d</b>", item->birthyear);
  g_string_append_printf (buf, "<j>%d</j>", item->job);
  g_string_append_printf (buf, "<r>%d</r>", item->look_for);
  g_string_append_printf (buf, "<g>%d</g>", item->voice);
  g_string_append_printf (buf, "<p>%d</p>", item->plans);
  buf = g_string_append (buf, "</query></iq>");
  q = g_string_free (buf, FALSE);
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &bytes);
  g_free (q);
  return ret;
}


/**
  * tlen_pubdir_get:
  * @tc: connection to use
  *
  * Get public information about yourself.
  *
  * returns: TRUE on success, FALSE otherwise.
  */
gboolean tlen_pubdir_get (TlenConnection *tc)
{
  gchar *q;
  gboolean ret;
  gsize bytes;

  if (!tc)
  {
    tlen_debug ("tc == NULL\n");
    return FALSE;
  }
  q = "<iq type='get' id='tr' to='tuba'><query xmlns='jabber:iq:register'></query></iq>";
  ret = tlen_connection_socket_writen (tc, q, tlen_strlen (q), &bytes);
  return ret;
}

TlenPubdirItem *tlen_pubdir_item_new (void)
{
  return  g_new0 (TlenPubdirItem, 1);
}

void tlen_pubdir_item_free (TlenPubdirItem *item)
{
  if (item)
  {
    g_free(item->firstname);
    g_free(item->lastname);
    g_free(item->nick);
    g_free(item->city);
    g_free(item->email);
    g_free(item->school);
    g_free(item->id);
    g_free(item);
  }
}
