"""
 Copyright by Krzysztof Stachlewski
 redistribution and use in source and binary forms, with or without modification,
 are permitted worldwide.

 MODULE:
     Prototype of lowlevel libtlen's interface.
   
"""

import string
import asyncore
import xml.sax
import urllib
import time
import threading
import unittest
import sha
import Queue

PRESENCE_AVAILABLE   = "available"
PRESENCE_AWAY        = "away"
PRESENCE_CHAT        = "chat"
PRESENCE_XA          = "xa"
PRESENCE_DND         = "dnd"
PRESENCE_INVISIBLE   = "invisible"
PRESENCE_UNAVAILABLE = "unavailable"

tlen_server_ip = "62.87.222.136"

class NoMoreEventsError(exceptions.Exception):
    pass

def EncodeText(text):
    return urllib.quote_plus(text)

def DecodeText(encoded_text):
    return urllib.unquote_plus(encoded_text)

klijent = TlenClient1()
klijent.FindServer()
while 1:
    klijent.Watch()
    try:
        while 1:
            event = klijent.GetNextEvent()
            # Process event.
    except NoMoreEventsError:
        pass
        
MAX_RECV_PART   = 1000
TCST_NEUTRAL    = 1
TCST_CONNECTING = 2
TCST_CONNECTED_MAYBE  = 3
TCST_CONNECTED  = 4
TCTS_ERROR      = 5

class XmlThread(threading.Thread):
    """A thread for XML parser.

    """
    def __init__(self, pip, xml_handler):
        super(XmlThread, self).__init__()
        self._Pipe = pip
        self._XmlHandler = xml_handler

    def run(self):
        xml.sax.parse(self._Pipe, self._XmlHandler)


class TlenClient1(dispatcher):
    """Level1 tlen client.
    
    """
    def __init__(self):
        self.State         = TCST_NEUTRAL
        self._Map          = {}
        self._SendBuffer   = ""
        self._EventsQ      = Queue.Queue(maxsize=0)      # For events coming from XmlThread.
        self.MaxRecvPart   = MAX_RECV_PART
        self._Pipe         = StringPipe()                # Connects sockets and XmlThread.
        self._XmlHandler   = XmlHandler(self._EventFun)  # Handles XML parser notifications on XmlThread.
        self._XmlThread    = XmlThread(self._Pipe, self._XmlHandler)
        super(TlenClient1, self).__init__(map = self._Map)
        self._XmlThread.start()

    def _EventFun(self, event):
        """Called whenever a new event arrives.

        Called on XmlThread from within XmlHandler.
        """
        self._EventsQ.put_nowait(event)

    def _IsConnected(self):
        return self.State == TCST_CONNECTED

#
# Asyncore handlers.
#

    def handle_read(self):
        """Data can be read from the socket.

        Called by asyncore.poll() which in turn is called by Watch().
        """
        try:
            new_data = self.recv(self.MaxRecvPart)
            all_data = new_data
            while len(new_data) == self.MaxRecvPart:
                new_data = self.recv(self.MaxRecvPart)
                all_data +=
        except socket.error, why:
            self.handle_close()
        else:
            if not new_data:
                # handle_close has already been called by asyncore.
                pass
            else:
                self._Pipe.push(new_data)
                if self.State == TCST_CONNECTED_MAYBE:
                    self.State = TCST_CONNECTED

    def handle_write(self):
        """Data can be written to the socket.

        Called by asyncore.poll().
        Tries to write everything from the internal buffers.
        """
        try:
            written = self.send(self._SendBuffer)
        except socket.error, why:
            self.handle_close()
        else:
            self._SendBuffer = self._SendBuffer[written:]
            if self.state == TCST_CONNECTED_MAYBE:
                self.State = TCST_CONNECTED
    
    def handle_connect(self):
        # Connected? Maybe. Maybe not.
        self.State  = TCST_CONNECTED_MAYBE
    
    def handle_close(self):
        self.close()
        self._Pipe.stop_pushing()
        self.State = TCST_NEUTRAL

    def readable(self):
        return 1

    def writable(self):
        return 1

#
# Interface.
#

    def Watch(self, timeout=1.0):
        """Contains asynchronous sockets magic that do all the work.

        """
        super(TlenClient1, self).poll(timeout, self._Map) # causes handle_read and handle_write to be called
        
    def GetNextEvent(self):
        """Returns next available event or raises NoMoreEventsError.
        
        """
        try:
            rc = self._EventsQ.get_nowait()
        except Queue.Empty:
            raise NoMoreEventsError()
        else:
            return rc

    def Connect(self, host, tlenid):
        """Connects to the given Tlen server.

        """
        self.State = TCST_CONNECTING
        try:
            self.connect(host)
        except socket.error, why:
            raise 

    def Disconnect(self):
        pass

    def Authorization(self, password):
        pass

    def RosterRequest(self):
        pass
    
    def Presence(self, presence):
        pass

    def Message(self, msg_type, userid, body):
        pass

    def Subscribe(self, userid):
        pass

    def RosterSet(self, userid):
        pass

    def RosterDel(self, userid):
        pass

    def Files(self, file_list):
        pass


class TlenEvent(object):
    def __init__(self):
        raise NotImplementedError()

class SocketErrorEvent(TlenEvent):
    """An error occured in the socket layer."""
    def __init__(self, why):
        self.why = why

class ConnectionEvent(TlenEvent):
    """Tlen server gave us the sessionid."""
    def __init__(self, sessionid):
        self.sessionid = sessionid

class AuthorizationEvent(TlenEvent):
    """Authorization accepted or declined."""
    def __init__(self, authorized, tlenid):
        self.authorized = authorized
        self.tlenid     = tlenid

class PresenceEvent(TlenEvent):
    """Someone has changed his own status."""
    def __init__(self, source, show, status):
        self.source = source
        self.show   = show
        self.status = status

class MessageEvent(TlenEvent):
    """A message from someone arrived."""
    def __init__(self, source, msgtype, body):
        self.source  = source
        self.msgtype = msgtype
        self.body    = body

class SubscribedEvent(TlenEvent):
    """Someone agreed to be on our list."""
    def __init__(self, source):
        self.source = source

class RosterEvent(TlenEvent):
    """An event related to the roster."""
    def __init__(self):
        raise NotImplementedError()

class RosterReceivedEvent(RosterEvent):
    """Address book received."""
    def __init__(self, items):
        self.items = items

class TlenErrorEvent(TlenEvent):
    def __init__(self):
        raise NotImplementedError()

class ProtocolBrokenEvent(TlenErrorEvent):
    """Communication is possibly broken."""
    pass

class ProtocolWarningEvent(TlenErrorEvent):
    """Some quirks detected that disallow interpretation of the
    current action. For instance an unknown tag has been received.
    Processing will continue but current action may be abandoned.
    """
    pass


class TlenClient(object):
    """
    """
    def __init__(self, tlenid, host='idi.tlen.pl'):
        pass

    def RequestConnection(self):
        """Connects to a server and initiates a session."""
        raise NotImplementedError()

    def NotifyConnection(self):
        """Notifies of the result of RequestConnection."""
        raise NotImplementedError()
      
    def RequestAuthorization(self, password):
        """Authorizes the user."""
        raise NotImplementedError()

    def NotifyAuthorization(self):
        raise NotImplementedError()

    def RequestRoster(self):
        raise NotImplementedError()

    def NotifyRoster(self):
        raise NotImplementedError()

    def RequestPresence(self):
        raise NotImplementedError()

    def NotifyRemotePresence(self):
        """Someone else has changed his own presence."""
        raise NotImplementedError()

    def RequestDisconnect(self):
        raise NotImplementedError()

    def NotifyDisconnect(self):
        raise NotImplementedError()


ST_NOT_STARTED          = -1
ST_NEUTRAL              = 0
ST_ROSTER               = 1
ST_ROSTER_QUERY         = 2
ST_ROSTER_ITEM          = 3
ST_ROSTER_GROUP         = 4
ST_PRESENCE             = 10
ST_SUBSCRIBED           = 11
ST_PRESENCE_SHOW        = 12
ST_PRESENCE_STATUS      = 13
ST_MESSAGE              = 20
ST_MESSAGE_BODY         = 21
ST_AUTHORIZATION        = 30
ST_SKIP                 = 100  # Skips until current tag is closed.
    
class XmlHandler(xml.sax.ContentHandler):
    """Handles tlen server output. Converts XML to simpler structures.

    Called from inside of xml.sax. Does not raise exceptions.
    Calls send_event_fun with appropriate events as parameters.
    This is a state machine, that knows only about the incoming data
    and its format but not the context.

    General behaviour:
     Ignores unknown data and tags.
     Does not detect multiple tags (where a single tag is expected).
     Most of events are sent when <iq>, <presence> or <message> tags are closed
     provided that enough data is collected for them to be valid.

    session_id - 8 character identifier of the session - a 32 bit number in hex
    tlenid     - a string of type "someone@tlen.pl"
    """
 
    def __init__(self, send_event_fun):
        self.state          = ST_NOT_STARTED
        self.send_event_fun = send_event_fun
        self.event_data     = {}               # Data for events is collected here.
        self.event_type     = None             # Set this for the SendEvent to know what kind of event we want to send.
        self.skip_nest      = 0
        self.skip_back_to   = None

        self.st_element = {
          ST_NOT_STARTED:       (self.StNotStarted,     None,                      None),
          ST_NEUTRAL:           (self.StNeutral,        self.StEndNeutral,         None),
          ST_ROSTER:            (self.StRoster,         self.StEndRoster,          None),
          ST_ROSTER_QUERY:      (self.StRosterQuery,    self.StEndRosterQuery,     None),
          ST_ROSTER_ITEM:       (self.StRosterItem,     self.StEndRosterItem,      None),
          ST_ROSTER_GROUP:      (None,                  self.StEndRosterGroup,     self.StRosterGroupChars),
          ST_PRESENCE:          (self.StPresence,       self.StEndPresence,        None),
          ST_PRESENCE_SHOW:     (None,                  self.StEndPresenceShow,    self.StPresenceShowChars),
          ST_PRESENCE_STATUS:   (None,                  self.StEndPresenceStatus,  self.StPresenceStatusChars),
          ST_MESSAGE:           (self.StMessage,        self.StEndMessage,         None),
          ST_MESSAGE_BODY:      (None,                  self.StEndMessageBody,     self.StMessageBodyChars),
          ST_AUTHORIZATION:     (None,                  self.StEndAuthorization,   None),
          ST_SUBSCRIBED:        (None,                  self.StEndSubscribed,      None),
          ST_SKIP:              (self.StSkip,           self.StEndSkip,            None)
        }

    #
    # Functions called by XML processing library.
    #

    def startElement(self, name, attrs):
        #print "\nSTART: %s@%d\n" % (name, self.state)
        if self.st_element[self.state][0]:
            self.st_element[self.state][0](name, attrs)
        else:
            self.Skip()
    
    def endElement(self, name):
        #print "\nEND: %s@%d\n" % (name, self.state)
        if self.st_element[self.state][1]: 
            self.st_element[self.state][1](name)
           
    def characters(self, data):
        if self.st_element[self.state][2]:
            self.st_element[self.state][2](data)

    def endDocument(self):
        """End of document - errorneous or regular one.

        Send a Closing event.
        """
        pass
    
    #
    # Implementation.
    #

    def SendEvent(self):
        """Sends an event.

        Uses self.event_type and self.event_data.
        """
        if self.event_data and self.event_type:
            try:
                event_to_send = self.event_type(**self.event_data)
            except TypeError:
                event_to_send = InternalErrorEvent(                                 \
                    "SendEvent: Not enough data for an event %s. event_data = %s" % \
                    (self.event_type.__name__, str(self.event_data)))
            self.send_event_fun(event_to_send)
            self.ClearEvent()

    def ClearEvent(self):
        self.event_data = {}
        self.event_type = None

    def InternalError(self, errstr):
        print errstr

    #
    # Skip.
    #

    def Skip(self, back_to = None):
        if back_to is None:
            back_to = self.state
        self.skip_back_to = back_to
        self.state        = ST_SKIP
        self.skip_nest    = 1

    def StSkip(self, name, attrs):
        self.skip_nest += 1

    def StEndSkip(self, name):
        self.skip_nest -= 1
        if self.skip_nest == 0:
            self.state = self.skip_back_to
            self.skip_nest    = None
            self.skip_back_to = None

    #
    # Neutral state.
    #
        
    def StNotStarted(self, name, attrs):
        if name == "s":
            try:
                vsessionid = attrs["i"]
            except KeyError:
                self.state = ST_SKIP
            else:
                try:
                    vsessionid_int  = int(vsessionid, 16)
                except ValueError:
                    self.Skip()
                else:
                    self.state      = ST_NEUTRAL
                    self.event_type = ConnectionEvent
                    self.event_data = {"sessionid": vsessionid}
                    self.SendEvent()
        else:
            self.state = ST_SKIP
    
    def StNeutral(self, name, attrs):
        handled = 0
        if name == "iq":
            try:
                vid   = attrs["id"]
                vtype = attrs["type"]
            except KeyError:
                pass
            else:
                try:
                    vsessionid = int(vid, 16)
                except ValueError:
                    if attrs.has_key("from") and vid == "GetRoster" and vtype == "result":
                        # Roster response.
                        handled         = 1
                        self.state      = ST_ROSTER
                        self.event_data = {}
                else:
                    # Could be an authorization response.
                    if vtype == "result":
                        # Authentication good.
                        handled         = 1
                        self.event_data = {"authorized": 1, "tlenid": vid}
                        self.state      = ST_AUTHORIZATION
                    elif vtype == "error":
                        # Authentication failed.
                        handled         = 1
                        self.event_data = {"authorized": 0, "tlenid": vid}
                        self.state      = ST_AUTHORIZATION
        elif name == "presence":
            try:
                vfrom = attrs["from"]
            except KeyError:
                pass
            else:
                try:
                    vtype = attrs["type"]
                except KeyError:
                    # Presence from someone.
                    handled          = 1
                    self.state       = ST_PRESENCE
                    self.event_data  = {"source": vfrom}
                else:
                    if vtype == "subscribed":
                        # Subscribed response.
                        handled         = 1
                        self.event_data = {"source": vfrom}
                        self.state      = ST_SUBSCRIBED
        elif name == "message":
            try:
                vfrom  = attrs["from"]
                vtype  = attrs["type"]
            except KeyError:
                pass
            else:
                if vtype in ("normal", "chat"):
                    handled = 1
                    self.event_data = {"source": vfrom, "msgtype": vtype}
                    self.state      = ST_MESSAGE
                    
        if not handled:
            self.Skip()

    def StEndNeutral(self, name):
        print "endneutral"
    
    #            
    # Message.
    #

    def StMessage(self, name, attrs):
        if name == "body":
            self.state = ST_MESSAGE_BODY
        else:
            self.Skip()

    def StEndMessage(self, name):
        if name != "message":
            self.InternalError("StEndMessage: Name is not 'message'")
        else:
            if "body" in self.event_data:
                self.event_type = MessageEvent
                self.SendEvent()
            else:
                self.ClearEvent()
            self.state = ST_NEUTRAL
            
    def StEndMessageBody(self, name):
        if name != "body":
            self.InternalError("StEndMessageBody: Name is not 'body'")
        else:
            self.state = ST_MESSAGE
            
    def StMessageBodyChars(self, data):
        self.event_data["body"] = self.event_data.get("body", "") + data

    #
    # Presence
    #

    def StPresence(self, name, attrs):
        if name == "show":
            self.state = ST_PRESENCE_SHOW
        elif name == "status":
            self.state = ST_PRESENCE_STATUS
        else:
            self.Skip()

    def StEndPresence(self, name):
        if name != "presence":
            self.InternalError("StEndPresence: Name is not 'presence'")
        else:
            if ("show" in self.event_data) and ("status" in self.event_data):
                self.event_type = PresenceEvent
                self.SendEvent()
                self.state = ST_NEUTRAL

    def StEndPresenceShow(self, name):
        if name != "show":
            self.InternalError("StEndPresenceShow: Name is not 'show'")
        else:
            self.state = ST_PRESENCE
        
    def StPresenceShowChars(self, data):
        self.event_data["show"] = self.event_data.get("show", "") + data

    def StEndPresenceStatus(self, name):
        if name != "status":
            self.InternalError("StEndPresenceStatus: Name is not 'status'")
        else:
            self.state = ST_PRESENCE

    def StPresenceStatusChars(self, data):
        self.event_data["status"] = self.event_data.get("status", "") + data

    #
    # Other
    #

    def StEndAuthorization(self, name):
        if name != "iq":
            self.InternalError("StEndAuthorization: Name is not 'iq'")
        else:
            self.event_type = AuthorizationEvent
            self.SendEvent()
            self.state = ST_NEUTRAL

    def StEndSubscribed(self, name):
        if name != "presence":
            self.InternalError("StEndSubscribed: Name is not 'presence'")
        else:
            self.event_type = SubscribedEvent
            self.SendEvent()
            self.state = ST_NEUTRAL        

    #
    # Roster
    #

    def StRoster(self, name, attrs):
        if name == "query":
            try:
                vxmlns = attrs["xmlns"]    
            except KeyError:
                self.Skip()
            else:
                if vxmlns == "jabber:iq:roster":
                    self.state      = ST_ROSTER_QUERY
                    self.event_data = {"items": []}
        else:
            self.Skip()

    def StEndRoster(self, name):
        if name != "iq":
            self.InternalError("StEndRoster: Name is not 'iq' but %s" % name)
        else:
            self.event_type = RosterReceivedEvent
            self.SendEvent()
            self.state = ST_NEUTRAL

    def StRosterQuery(self, name, attrs):
        if name == "item":
            try:
                vjid          = attrs["jid"]
                vname         = attrs["name"]
                vsubscription = attrs["subscription"]
            except KeyError:
                self.Skip()
            else:
                self.event_data["new_item"] = {"tlenid": vjid,
                                               "name":   vname,
                                               "subscription": vsubscription}
                self.state = ST_ROSTER_ITEM
        else:
            self.Skip()

    def StEndRosterQuery(self, name):
        if name != "query":
            self.InternalError("StEndRosterQuery: Name is not 'query' but %s" % name)
        else:
            self.state = ST_ROSTER

    def StRosterItem(self, name, attrs):
        if name == "group":
            self.state = ST_ROSTER_GROUP
        else:
            self.Skip()

    def StEndRosterItem(self, name):
        if name != "item":
            self.InternalError("StEndRosterItem: Name is not 'item'")
        else:
            if self.event_data["new_item"].has_key("group"):
                self.event_data["items"].append(self.event_data["new_item"])
                del self.event_data["new_item"]
            self.state = ST_ROSTER_QUERY

    def StEndRosterGroup(self, name):
        if name != "group":
            self.InternalError("StEndRosterGroup: Name is not 'group'")
        else:
            self.state = ST_ROSTER_ITEM
            
    def StRosterGroupChars(self, data):
        self.event_data["new_item"]["group"] = self.event_data["new_item"].get("group", "") + data

        
class StringPipe(object):
    """Implements a file-like pipe object.

    This class is suitable for a setup where one thread reads
    from the file object and another thread successively
    pushes data into it.
    """
    def __init__(self):
        self.monitor = threading.Condition()
        self.eof = 0
        self.buf = ""

    def push_bytes(self, bytes):
        """Adds bytes to the internal buffer.
        
        Interface for pushing thread.
        """
        self.monitor.acquire()
        self.buf += bytes
        self.monitor.notify()
        self.monitor.release()

    def stop_pushing(self):
        """Puts EOF marker.
        
        Interface for pushing thread.
        Pushing cannot be restarted.
        """
        self.monitor.acquire()
        self.eof = 1
        self.monitor.notify()
        self.monitor.release()

    def close(self):
        """Closes file.

        Interface for reading thread.
        """
        return

    def flush(self):
        return

    def read(self, n=None):
        """Reads n bytes from internal buffer.

        """
        self.monitor.acquire()
        rc = ""
        if self.eof and not self.buf:
            pass # Signal EOF.
        else:
            while (not self.buf) and (not self.eof):
                self.monitor.wait()
            # At this moment some data is available or EOF marker is set or both.
            if self.buf:
                if n is None:
                    rc       = self.buf
                    self.buf = ""
                else:
                    if len(self.buf) > n:
                        rc       = self.buf[:n]
                        self.buf = self.buf[n:]
                    else:
                        rc       = self.buf
                        self.buf = ""
            else:
                pass # Signal EOF.
        self.monitor.release()
        return rc


def CalcTlenHash(passcode, tlenid):
    """Calculates hash needed to login to Tlen server.

    """
    magic1 = 0x50305735L
    magic2 = 0x12345671L
    sum    = 7
    for z in passcode:
        if (z == ' ') or (z == '\t'):
            continue
        magic1 ^= (((magic1 & 0x3f) + sum) * ord(z)) + ((magic1 << 8) & 0xFFFFFFFFL)
        magic2 += ((magic2 << 8) & 0xFFFFFFFFL) ^ magic1
        sum += ord(z)
    magic1 &= 0x7FFFFFFFL
    magic2 &= 0x7FFFFFFFL
    to_hash = "%s%08x%08x" % (tlenid, magic1, magic2)
    return sha.sha(to_hash[:24]).hexdigest()



#--------------------------------------#
#               UNITTEST               #
#--------------------------------------#

#
# Pipe test.
#

class TestStringPipe(unittest.TestCase):
    def testFlat(self):
        s = "<a></a>"
        pip = StringPipe()
        pip.push_bytes(s[:2])
        pip.push_bytes(s[2:])
        pip.stop_pushing()
        r = pip.read()
        self.assertEqual(r, s)


#
# Pipe test with XML and private XML handler.
#

class TestPrivateXmlHandler(xml.sax.ContentHandler):
    """XML handler for testing.

    Works only for simple elements without arguments.
    Outputs unicode string.
    """
    def __init__(self, output):
        """Init

        'output' should be a list with a single ustring element where the xml will be recreated."""
        self.output = output

    def startElement(self, name, attrs):
        self.output[0] += u"<%s>" % name
    
    def endElement(self, name):
        self.output[0] += u"</%s>" % name

    def characters(self, data):
        self.output[0] += data


class TestPrivateXmlTask(threading.Thread):
    """This task starts the parser."""
    def __init__(self, pip, xml_handler):
        super(TestPrivateXmlTask, self).__init__()
        self.pip = pip
        self.xml_handler = xml_handler

    def run(self):
        xml.sax.parse(self.pip, self.xml_handler)


class TestXml(unittest.TestCase):
    def testPipeOnce(self):
        xml_str = u"""<a></a>"""
        output = [""]
        pip = StringPipe()
        xml_handler = TestPrivateXmlHandler(output)
        xml_handle_task = TestPrivateXmlTask(pip, xml_handler)
        xml_handle_task.start()

        pip.push_bytes(xml_str)
        pip.stop_pushing()
        time.sleep(1)
        self.assertEqual(xml_str, output[0])

    def testPipeTwice(self):
        xml_str = u"""<a><b>ALA</b></a>"""
        br = 6
        output = [""]
        pip = StringPipe()
        xml_handler = TestPrivateXmlHandler(output)
        xml_handle_task = TestPrivateXmlTask(pip, xml_handler)
        xml_handle_task.start()

        pip.push_bytes(xml_str[:br])
        time.sleep(1)   # poor man's synchronization
        self.assertEqual(xml_str[:br], output[0])

        pip.push_bytes(xml_str[br:])
        pip.stop_pushing()
        time.sleep(1)   # poor man's synchronization
        self.assertEqual(xml_str, output[0])

#
# XmlHandler test.
#

class TestXmlHandler(unittest.TestCase):
    def EventFun(self, event):
        print str(event), event.__dict__
        
    def testValidTlenXml(self):
        xml_str = """<s i="12341234">
        <iq type='result' id='3FBA8D6F'/>
        <iq type='error' id='3FBA8D6F'/>
        <iq type='result' id='GetRoster' from='matijozef@tlen.pl/t'>
         <query xmlns='jabber:iq:roster'>
           <item jid='libtlendev@tlen.pl' name='libtlendev' subscription='both'>
             <group>grupa</group>
           </item>
         </query>
        </iq>
        <presence from='libtlendev@tlen.pl' type='subscribed'/>
        <presence from='identyfikator@tlen.pl/t'><show>huha!</show><status>subscribed</status></presence>
        </s>
        """
        print
        xml_handler = XmlHandler(self.EventFun)
        pip = StringPipe()
        pip.push_bytes(xml_str)
        pip.stop_pushing()
        xml.sax.parse(pip, xml_handler)


if __name__=="__main__":
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestXmlHandler))
    suite.addTest(unittest.makeSuite(TestStringPipe))
    suite.addTest(unittest.makeSuite(TestXml))
    unittest.TextTestRunner(verbosity=2).run(suite)
